'' =================================================================================================
''
''   File....... jm_quad_pulse.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2020 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon.mcphalen@gmail.com
''   Started....
''   Updated.... 12 JUN 2020
''
''   {$P1} 
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  BR_TERM  = 115_200                                            ' terminal baud rate
  

con { io pins }                                                  
                                                                 
  RX1  = 31  { I }                                              ' serial / programming
  TX1  = 30  { O }                                            
                                                             
  SDA1 = 29  { I/O }                                            ' i2c / eeprom
  SCL1 = 28  { I/O } 


con

  #true,  ON, OFF 
  #false, NO, YES


obj

' main                                                          ' * master Spin cog
  time   : "jm_time_80"                                           '   timing and delays (80MHz)
  io     : "jm_io_basic"                                          '   essential io
  term   : "jm_fullduplexserial"                                  ' * serial IO for terminal
' pulses
                                                                 
' * uses cog when loaded

                                                                 
dat  


var

                                                       
pub main
                                                                 
  setup

  set_width(0, 1500)
  set_width(1, 1800)
  set_width(2, 2200)
  set_width(3, 2500)      
                                                                 
  repeat                                                         
    waitcnt(0)


pub wait_for_terminal(clear)

  ' Download to RAM with F10
  ' F12 to open PST
  ' Click [Enable]
  ' Press Enter

  term.rxflush
  term.rx
  if (clear)
    term.tx(term#CLS)

                                                               
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)
                                                                 
  term.start(RX1, TX1, %0000, BR_TERM)                          ' start serial for terminal *

  start_pulse_outs                                              ' start pulse outputs


con

  ' Pulse output pins
  ' -- code expects contiguous group from base

  PULSE_3 = 15                                                  ' pulse output pins
  PULSE_2 = 14
  PULSE_1 = 13
  PULSE_0 = 12                                                  ' base pin


var

  long  postack[32]                                             ' stack for Spin cog

  long  width0                                                  ' pulse width
  long  width1
  long  width2
  long  width3


pub start_pulse_outs

  longfill(@width0, 2000 * US_001, 4)                           ' default to midpoint

  cognew(pulse_outs, @postack)                                  ' start the cog 


pub set_width(ch, us)

'' Set channel pulse in microseconds
'' -- ch is 0..3
'' -- us is 1500..2500 (1.5 to 2.5ms)

  if ((ch => 0) and (ch =< 3))                                  ' good channel?
    width0[ch] := (1500 #> us <# 2500) * US_001                 ' convert us to ticks
 

pri pulse_outs | t, ch

  t := cnt
  repeat
    repeat ch from 0 to 3                                       ' loop through channels
      ctra := (%00100 << 26) | (PULSE_0 + ch)                   ' set to NCO mode
      frqa := 1
      phsa := -width0[ch]                                       ' start new pulse
      dira[PULSE_0 + ch] := 1                                   ' make pin an output
      waitcnt(t += constant(5 * MS_001))                        ' let hold for pulse window

         
con { license }                                                  

{{

  MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:
  
  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.
  
  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.

}}     